import io
import os

class BingTranscriber:
    def transcribe(self, audio_path):
        pass

    def __init__(self, output_control, audio_settings_dict, expected_calls):
        self.output_control = output_control
